@@ -1,5 +1,6 @@
 #include "grid.h"
 
+#include <limits.h>
 #include <stdlib.h>
 #include <string.h>
 
@@ -16,7 +17,7 @@
 #define TIME_REFLOW 0
 
 /*
- * “sb” (scrollback relative) coordinates
+ * "sb" (scrollback relative) coordinates
  *
  * The scrollback relative row number 0 is the *first*, and *oldest*
  * row in the scrollback history (and thus the *first* row to be
@@ -231,7 +232,7 @@ grid_snapshot(const struct grid *grid)
         clone_row->cells = xmalloc(grid->num_cols * sizeof(clone_row->cells[0]));
         clone_row->linebreak = row->linebreak;
         clone_row->dirty = row->dirty;
-        clone_row->prompt_marker = row->prompt_marker;
+        clone_row->shell_integration = row->shell_integration;
 
         for (int c = 0; c < grid->num_cols; c++)
             clone_row->cells[c] = row->cells[c];
@@ -262,8 +263,7 @@ grid_snapshot(const struct grid *grid)
         int original_stride = stride_for_format_and_width(original_pix_fmt, original_width);
 
         size_t original_size = original_stride * original_height;
-        void *new_original_data = xmalloc(original_size);
-        memcpy(new_original_data, it->item.original.data, original_size);
+        void *new_original_data = xmemdup(it->item.original.data, original_size);
 
         pixman_image_t *new_original_pix = pixman_image_create_bits_no_clear(
             original_pix_fmt, original_width, original_height,
@@ -283,8 +283,7 @@ grid_snapshot(const struct grid *grid)
             int scaled_stride = stride_for_format_and_width(scaled_pix_fmt, scaled_width);
 
             size_t scaled_size = scaled_stride * scaled_height;
-            new_scaled_data = xmalloc(scaled_size);
-            memcpy(new_scaled_data, it->item.scaled.data, scaled_size);
+            new_scaled_data = xmemdup(it->item.scaled.data, scaled_size);
 
             new_scaled_pix = pixman_image_create_bits_no_clear(
                 scaled_pix_fmt, scaled_width, scaled_height, new_scaled_data,
@@ -366,7 +365,9 @@ grid_row_alloc(int cols, bool initialize)
     row->dirty = false;
     row->linebreak = false;
     row->extra = NULL;
-    row->prompt_marker = false;
+    row->shell_integration.prompt_marker = false;
+    row->shell_integration.cmd_start = -1;
+    row->shell_integration.cmd_end = -1;
 
     if (initialize) {
         row->cells = xcalloc(cols, sizeof(row->cells[0]));
@@ -425,7 +426,9 @@ grid_resize_without_reflow(
 
         new_row->dirty = old_row->dirty;
         new_row->linebreak = false;
-        new_row->prompt_marker = old_row->prompt_marker;
+        new_row->shell_integration.prompt_marker = old_row->shell_integration.prompt_marker;
+        new_row->shell_integration.cmd_start = min(old_row->shell_integration.cmd_start, new_cols - 1);
+        new_row->shell_integration.cmd_end = min(old_row->shell_integration.cmd_end, new_cols - 1);
 
         if (new_cols > old_cols) {
             /* Clear "new" columns */
@@ -587,7 +590,9 @@ _line_wrap(struct grid *old_grid, struct row **new_grid, struct row *row,
         /* Scrollback is full, need to reuse a row */
         grid_row_reset_extra(new_row);
         new_row->linebreak = false;
-        new_row->prompt_marker = false;
+        new_row->shell_integration.prompt_marker = false;
+        new_row->shell_integration.cmd_start = -1;
+        new_row->shell_integration.cmd_end = -1;
 
         tll_foreach(old_grid->sixel_images, it) {
             if (it->item.pos.row == *row_idx) {
@@ -599,7 +604,7 @@ _line_wrap(struct grid *old_grid, struct row **new_grid, struct row *row,
         /*
          * TODO: detect if the reused row is covered by the
          * selection. Of so, cancel the selection. The problem: we
-         * don’t know if we’ve translated the selection coordinates
+         * don't know if we've translated the selection coordinates
          * yet.
          */
     }
@@ -609,7 +614,7 @@ _line_wrap(struct grid *old_grid, struct row **new_grid, struct row *row,
         return new_row;
 
     /*
-     * URI ranges are per row. Thus, we need to ‘close’ the still-open
+     * URI ranges are per row. Thus, we need to 'close' the still-open
      * ranges on the previous row, and re-open them on the
      * next/current row.
      */
@@ -789,7 +794,7 @@ grid_resize_and_reflow(
         }
 
         if (!old_row->linebreak && col_count > 0) {
-            /* Don’t truncate logical lines */
+            /* Don't truncate logical lines */
             col_count = old_cols;
         }
 
@@ -831,35 +836,26 @@ grid_resize_and_reflow(
             int end;
             bool tp_break = false;
             bool uri_break = false;
+            bool ftcs_break = false;
 
-            /*
-             * Set end-coordinate for this chunk, by finding the next
-             * point-of-interest on this row.
-             *
-             * If there are no more tracking points, or URI ranges,
-             * the end-coordinate will be at the end of the row,
-             */
-            if (range != range_terminator) {
-                int uri_col = (range->start >= start ? range->start : range->end) + 1;
+            /* Figure out where to end this chunk */
+            {
+                const int uri_col = range != range_terminator
+                    ? ((range->start >= start ? range->start : range->end) + 1)
+                    : INT_MAX;
+                const int tp_col = tp != NULL ? tp->col + 1 : INT_MAX;
+                const int ftcs_col = old_row->shell_integration.cmd_start >= start
+                    ? old_row->shell_integration.cmd_start + 1
+                    : old_row->shell_integration.cmd_end >= start
+                    ? old_row->shell_integration.cmd_end + 1
+                    : INT_MAX;
 
-                if (tp != NULL) {
-                    int tp_col = tp->col + 1;
-                    end = min(tp_col, uri_col);
+                end = min(col_count, min(min(tp_col, uri_col), ftcs_col));
 
-                    tp_break = end == tp_col;
-                    uri_break = end == uri_col;
-                    LOG_DBG("tp+uri break at %d (%d, %d)", end, tp_col, uri_col);
-                } else {
-                    end = uri_col;
-                    uri_break = true;
-                    LOG_DBG("uri break at %d", end);
-                }
-            } else if (tp != NULL) {
-                end = tp->col + 1;
-                tp_break = true;
-                LOG_DBG("TP break at %d", end);
-            } else
-                end = col_count;
+                uri_break = end == uri_col;
+                tp_break = end == tp_col;
+                ftcs_break = end == ftcs_col;
+            }
 
             int cols = end - start;
             xassert(cols > 0);
@@ -887,8 +883,8 @@ grid_resize_and_reflow(
                 xassert(amount > 0);
 
                 /*
-                 * If we’re going to reach the end of the new row, we
-                 * need to make sure we don’t end in the middle of a
+                 * If we're going to reach the end of the new row, we
+                 * need to make sure we don't end in the middle of a
                  * multi-column character.
                  */
                 int spacers = 0;
@@ -897,7 +893,7 @@ grid_resize_and_reflow(
                      * While the cell *after* the last cell is a CELL_SPACER
                      *
                      * This means we have a multi-column character
-                     * that doesn’t fit on the current row. We need to
+                     * that doesn't fit on the current row. We need to
                      * push it to the next row, and insert CELL_SPACER
                      * cells as padding.
                      */
@@ -920,7 +916,7 @@ grid_resize_and_reflow(
                 xassert(from + amount <= old_cols);
 
                 if (from == 0)
-                    new_row->prompt_marker = old_row->prompt_marker;
+                    new_row->shell_integration.prompt_marker = old_row->shell_integration.prompt_marker;
 
                 memcpy(
                     &new_row->cells[new_col_idx], &old_row->cells[from],
@@ -979,6 +975,16 @@ grid_resize_and_reflow(
                 }
             }
 
+            if (ftcs_break) {
+                xassert(old_row->shell_integration.cmd_start == start + cols - 1 ||
+                        old_row->shell_integration.cmd_end == start + cols - 1);
+
+                if (old_row->shell_integration.cmd_start == start + cols - 1)
+                    new_row->shell_integration.cmd_start = new_col_idx - 1;
+                if (old_row->shell_integration.cmd_end == start + cols - 1)
+                    new_row->shell_integration.cmd_end = new_col_idx - 1;
+            }
+
             left -= cols;
             start += cols;
         }
@@ -996,9 +1002,9 @@ grid_resize_and_reflow(
             {
                 /*
                  * line_wrap() "closes" still-open URIs. Since this is
-                 * the *last* row, and since we’re line-breaking due
+                 * the *last* row, and since we're line-breaking due
                  * to a hard line-break (rather than running out of
-                 * cells in the "new_row"), there shouldn’t be an open
+                 * cells in the "new_row"), there shouldn't be an open
                  * URI (it would have been closed when we reached the
                  * end of the URI while reflowing the last "old"
                  * row).
@@ -1025,7 +1031,7 @@ grid_resize_and_reflow(
     xassert(old_rows == 0 || *next_tp == &terminator);
 
 #if defined(_DEBUG)
-    /* Verify all URI ranges have been “closed” */
+    /* Verify all URI ranges have been "closed" */
     for (int r = 0; r < new_rows; r++) {
         const struct row *row = new_grid[r];
 
@@ -1069,7 +1075,7 @@ grid_resize_and_reflow(
     grid->num_cols = new_cols;
 
     /*
-     * Set new viewport, making sure it’s not too far down.
+     * Set new viewport, making sure it's not too far down.
      *
      * This is done by using scrollback-start relative cooardinates,
      * and bounding the new viewport to (grid_rows - screen_rows).
@@ -1135,7 +1141,7 @@ grid_row_uri_range_put(struct row *row, int col, const char *uri, uint64_t id)
         const bool matching_id = r->id == id;
 
         if (matching_id && r->end + 1 == col) {
-            /* Extend existing URI’s tail */
+            /* Extend existing URI's tail */
             r->end++;
             goto out;
         }
@@ -1174,7 +1180,7 @@ grid_row_uri_range_put(struct row *row, int col, const char *uri, uint64_t id)
                 uri_range_insert(extra, i + 1, col + 1, r->end, r->id, r->uri);
 
                 /* The insertion may xrealloc() the vector, making our
-                 * ‘old’ pointer invalid */
+                 * 'old' pointer invalid */
                 r = &extra->uri_ranges.v[i];
                 r->end = col - 1;
                 xassert(r->start <= r->end);
@@ -1311,7 +1317,7 @@ grid_row_uri_range_erase(struct row *row, int start, int end)
                 extra, i + 1, end + 1, old->end, old->id, old->uri);
 
             /* The insertion may xrealloc() the vector, making our
-             * ‘old’ pointer invalid */
+             * 'old' pointer invalid */
             old = &extra->uri_ranges.v[i];
             old->end = start - 1;
             return;  /* There can be no more URIs affected by the erase range */
@@ -1394,11 +1400,11 @@ UNITTEST
      * The insertion logic typically triggers an xrealloc(), which, in
      * some cases, *moves* the entire URI vector to a new base
      * address. grid_row_uri_range_erase() did not account for this,
-     * and tried to update the ‘end’ member in the URI range we just
+     * and tried to update the 'end' member in the URI range we just
      * split. This causes foot to crash when the xrealloc() has moved
      * the URI range vector.
      *
-     * (note: we’re only verifying we don’t crash here, hence the lack
+     * (note: we're only verifying we don't crash here, hence the lack
      * of assertions).
      */
     free(row_data.uri_ranges.v);
