@@ -20,7 +20,7 @@ notify_notify(const struct terminal *term, const char *title, const char *body)
     LOG_DBG("notify: title=\"%s\", msg=\"%s\"", title, body);
 
     if (term->conf->notify_focus_inhibit && term->kbd_focus) {
-        /* No notifications while we’re focused */
+        /* No notifications while we're focused */
         return;
     }
 
@@ -36,7 +36,7 @@ notify_notify(const struct terminal *term, const char *title, const char *body)
     if (!spawn_expand_template(
             &term->conf->notify, 4,
             (const char *[]){"app-id", "window-title", "title", "body"},
-            (const char *[]){term->conf->app_id, term->window_title, title, body},
+            (const char *[]){term->app_id ? term->app_id : term->conf->app_id, term->window_title, title, body},
             &argc, &argv))
     {
         return;
