@@ -1374,17 +1374,12 @@ void
 search_input(struct seat *seat, struct terminal *term,
              const struct key_binding_set *bindings, uint32_t key,
              xkb_keysym_t sym, xkb_mod_mask_t mods, xkb_mod_mask_t consumed,
-             xkb_mod_mask_t locked,
              const xkb_keysym_t *raw_syms, size_t raw_count,
              uint32_t serial)
 {
     LOG_DBG("search: input: sym=%d/0x%x, mods=0x%08x, consumed=0x%08x",
             sym, sym, mods, consumed);
 
-    const xkb_mod_mask_t bind_mods =
-        mods & seat->kbd.bind_significant & ~locked;
-    const xkb_mod_mask_t bind_consumed =
-        consumed & seat->kbd.bind_significant & ~locked;
     enum xkb_compose_status compose_status = seat->kbd.xkb_compose_state != NULL
       ? xkb_compose_state_get_status(seat->kbd.xkb_compose_state)
       : XKB_COMPOSE_NOTHING;
@@ -1399,7 +1394,7 @@ search_input(struct seat *seat, struct terminal *term,
 
         /* Match translated symbol */
         if (bind->k.sym == sym &&
-            bind->mods == (bind_mods & ~bind_consumed)) {
+            bind->mods == (mods & ~consumed)) {
 
             if (execute_binding(seat, term, bind, serial,
                                 &update_search_result, &search_direction,
@@ -1410,7 +1405,7 @@ search_input(struct seat *seat, struct terminal *term,
             return;
         }
 
-        if (bind->mods != bind_mods || bind_mods != (mods & ~locked))
+        if (bind->mods != mods)
             continue;
 
         /* Match untranslated symbols */
